grammar nameana;

options {
    output=ASTnull;
}

tokens {
	IDENTIFIER;
	STRING;
}

@members {
	SymbolTable symbolTable = new SymbolTable();
	NameTable   nametable   = new NameTable();
	
	private void printDefinition(Symbol symbol) {
		Definition definition = nametable.getCurrentDefinition(symbol);
		if (definition == null) {
			System.out.println("Error: " + symbol.toString() + " is not defined.");
		} else {
			System.out.println(symbol.toString() + " is " + definition.getValue());
		}
	}
	
	private void enterDefinition(Symbol symbol, String value) {
		if (nametable.definitionInCurrentScope(symbol)) {
			System.out.println("Error: multiple definitions of " + symbol.toString());
			return;
		}
		Definition definition = new Definition(value.substring(1, value.length() - 1));
		nametable.enterDefinition(symbol, definition);
	}
}

translationUnit : statement* ;
			
statement :
	  '{' { nametable.enterScope(); } statement* '}' { nametable.leaveScope(); }
	| sym=symbol '=' s=STRING ';'
		{ enterDefinition(sym, s.getText()); }
	| 'print' '(' sym=symbol ')' ';'
		{ printDefinition(sym); }	
;

symbol returns [Symbol symbol]: i=IDENTIFIER
	{ symbol = symbolTable.findOrInsert(i.getText()); }
;

WS : ( ' ' | '\t' | '\n' | '\r' )*
    { skip(); }
;

IDENTIFIER : ('a'..'z' | 'A'..'Z' | '_' | '0'..'9')+ ; 
STRING : ('\"' (~'\"')* '\"') ;